@@ -1,12 +1,12 @@
 package File::Sticker::Reader;
-$File::Sticker::Reader::VERSION = '1.0603';
+$File::Sticker::Reader::VERSION = '3.0005';
 =head1 NAME
 
 File::Sticker::Reader - read and standardize meta-data from files
 
 =head1 VERSION
 
-version 1.0603
+version 3.0005
 
 =head1 SYNOPSIS
 
@@ -99,7 +99,7 @@ sub init {
     {
 	$self->{$key} = $parameters{$key};
     }
-    $self->{file_magic} = File::LibMagic->new();
+    $self->{file_magic} = File::LibMagic->new(follow_symlinks=>1);
 } # init
 
 =head2 name
