@@ -1,12 +1,12 @@
 package File::Sticker::Writer::Exif;
-$File::Sticker::Writer::Exif::VERSION = '1.0603';
+$File::Sticker::Writer::Exif::VERSION = '3.0005';
 =head1 NAME
 
 File::Sticker::Writer::Exif - write and standardize meta-data from EXIF file
 
 =head1 VERSION
 
-version 1.0603
+version 3.0005
 
 =head1 SYNOPSIS
 
@@ -165,14 +165,41 @@ sub replace_one_field {
     }
     elsif ($field eq 'description')
     {
-        if ($ft->{mime_type} =~ /image\jpeg/)
+        # Okay, here's the messy relationship between the description,
+        # the freeform data, and GIMP.
+        #
+        # I originally wrote the freeform data in the UserComment field, and
+        # the description into the Description and Comment fields, then I found
+        # that GIMP uses the UserComment field as the description field at a
+        # higher priority than the Comment field.
+        #
+        # So then I wrote the freeform data in the ImageDescription field,
+        # then I found that GIMP ALSO uses THAT field as the description field
+        # at a higher priority than the Comment field.
+        #
+        # And that GIMP overwrites ALL THREE fields (Comment, UserComment, 
+        # ImageDescription) with what it considers the description
+        # when saving a file.
+        #
+        # So now I use the XMP:Description (Description) field for the
+        # freeform data, because GIMP neither reads nor overwrites that.
+        # WHEW!
+
+        # Before the decription is written, the freeform data
+        # needs to be converted to its new home.
+        $self->_convert_freeform_data(exif=>$et);
+
+        # GIMP reads and overwrites the Comment, UserComment
+        # and ImageDescription fields, so we need to do that too.
+        # Escpecially since GIMP does not look at the Comment field
+        # if one of the other two is not empty.
+        $success = $et->SetNewValue('UserComment', $value);
+        $success = $et->SetNewValue('ImageDescription', $value);
+        
+        if ($ft->{mime_type} =~ /image\/jpeg/)
         {
             $success = $et->SetNewValue('Comment', $value);
         }
-        else
-        {
-            $success = $et->SetNewValue('Description', $value);
-        }
     }
     elsif ($field eq 'tags')
     {
@@ -235,14 +262,14 @@ sub delete_field_from_file {
     }
     elsif ($field eq 'description')
     {
+        # GIMP reads and overwrites the Comment, UserComment
+        # and ImageDescription fields, so we need to do that too.
         if ($ft->{mime_type} =~ /image\/jpeg/)
         {
             $success = $et->SetNewValue('Comment');
         }
-        else
-        {
-            $success = $et->SetNewValue('Description');
-        }
+        $success = $et->SetNewValue('ImageDescription');
+        $success = $et->SetNewValue('UserComment');
     }
     elsif ($field eq 'tags')
     {
@@ -285,7 +312,7 @@ sub _get_the_real_file {
     {
         my $cover_file = ($self->{cover_file} ? $self->{cover_file} : 'cover.jpg');
         $cover_file = File::Spec->catfile($filename, $cover_file);
-        if (-f $cover_file)
+        if (-r $cover_file)
         {
             $filename = $cover_file;
         }
@@ -295,8 +322,8 @@ sub _get_the_real_file {
 
 =head2 _read_freeform_data
 
-Read the freeform data as YAML data from the UserComment field
-    
+Read the freeform data as YAML data from the XMP:Description field.
+ 
     my $ydata = $self->_read_freeform_data(exif=>$exif);
 
 =cut
@@ -306,17 +333,20 @@ sub _read_freeform_data {
     my %args = @_;
     say STDERR whoami() if $self->{verbose} > 2;
 
+    # CONVERT FREEFORM DATA if needed BEFOREHAND
+    $self->_convert_freeform_data(%args);
+
     my $ydata;
     my $et = $args{exif};
-    my $ystring = $et->GetValue('UserComment');
-    $ystring = $et->GetNewValue('UserComment') if !$ystring;
+    my $ystring = $et->GetValue('Description');
+    $ystring = $et->GetNewValue('Description') if !$ystring;
     say STDERR "ystring=$ystring" if $self->{verbose} > 2;
-    if ($ystring)
+    if ($ystring and $ystring =~ /^---/) # YAML data needs prefix
     {
         eval {$ydata = Load($ystring);};
         if ($@)
         {
-            warn __PACKAGE__, " Load of YAML data failed: $@";
+            warn __PACKAGE__, " Load of YAML data failed: $@ // '$ystring'";
         }
         elsif (!$ydata)
         {
@@ -329,10 +359,10 @@ sub _read_freeform_data {
 
 =head2 _write_freeform_data
 
-Write the freeform data as YAML data from the UserComment field
+Write the freeform data as YAML data into the Description field
 This overwrites whatever is there, it does not check.
     
-    $self->_read_freeform_data(newdata=>\%newdata,exif=>$exif);
+    $self->_write_freeform_data(newdata=>\%newdata,exif=>$exif);
 
 =cut
 
@@ -357,10 +387,97 @@ sub _write_freeform_data {
     }
     my $ystring = Dump($newdata);
     say STDERR "ystring=$ystring" if $self->{verbose} > 2;
-    my $success = $et->SetNewValue('UserComment', $ystring);
+    my $success = $et->SetNewValue('Description', $ystring);
     return $success;
 } # _write_freeform_data
 
+=head2 _convert_freeform_data
+
+Convert the freeform data so that it is placed into the Description field
+rather than the UserComment or ImageDescription field.
+ 
+    $self->_convert_freeform_data(exif=>$exif);
+
+=cut
+
+sub _convert_freeform_data {
+    my $self = shift;
+    my %args = @_;
+    say STDERR whoami() if $self->{verbose} > 2;
+
+    my $et = $args{exif};
+    # Check if it needs conversion at all.
+    # If the XMP:Description field is not empty
+    # and contains YAML data, then nothing needs to be done.
+    my $ystring = $et->GetValue('Description');
+    $ystring = $et->GetNewValue('Description') if !$ystring;
+    if ($ystring and $ystring =~ /^---/) # Assume YAML data
+    {
+        # no conversion needed
+        return 1;
+    }
+
+    # ------------------------------------
+    # Conversion needed
+    # Read from ImageDescription, write into XMP:Description
+    # The YAML data might be in UserComment instead if old.
+    # ------------------------------------
+    $ystring = $et->GetValue('ImageDescription') if !$ystring;
+    $ystring = $et->GetNewValue('ImageDescription') if !$ystring;
+    my $ydata;
+    my $success = 0;
+    if (!$ystring or $ystring =~ /^---/) # Try UserComment
+    {
+        $ystring = $et->GetValue('UserComment') if !$ystring;
+        $ystring = $et->GetNewValue('UserComment') if !$ystring;
+    }
+    if ($ystring and $ystring =~ /^---/) # Probably YAML data
+    {
+        # Check if the YAML data is valid
+        eval {$ydata = Load($ystring);};
+        if ($@)
+        {
+            warn __PACKAGE__, " Load of YAML data failed: $@ ## '$ystring'";
+        }
+        elsif (!$ydata)
+        {
+            warn __PACKAGE__, " no legal YAML" if $self->{verbose} > 1;
+        }
+        else # data is okay
+        {
+            $success = $et->SetNewValue('Description', $ystring);
+            if ($success)
+            {
+                # Put the description, if there is one,
+                # into the UserComment, ImageDescription
+                my $desc = $et->GetValue('Comment');
+                $desc = $et->GetNewValue('Comment') if !$desc;
+                $desc = $et->GetValue('Caption-Abstract') if !$desc;
+                if ($desc)
+                {
+                    $et->SetNewValue('UserComment', $desc);
+                    $et->SetNewValue('ImageDescription', $desc);
+                }
+                else
+                {
+                    # Otherwise clear out no longer valid data
+                    $et->SetNewValue('ImageDescription');
+                    $et->SetNewValue('UserComment');
+                }
+            }
+        }
+    }
+    else # No YAML data to convert
+    {
+        # Put some empty data in there.
+        my %newdata = ();
+        my $nystring = Dump(\%newdata);
+        $success = $et->SetNewValue('Description', $nystring);
+    }
+
+    return $success;
+} # _convert_freeform_data
+
 =head1 BUGS
 
 Please report any bugs or feature requests to the author.
